﻿using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Presentation;
using Microsoft.SPOT.Presentation.Media;
using System.Threading;

namespace BigDigitsWatchface
{
    public class Program
    {
        const int SCREEN_WIDTH = 128;
        const int SCREEN_HEIGHT = 128;
        const int DIGIT_WIDTH = 64;
        const int DIGIT_HEIGHT = 64;

        // set the following to true for 24-hour time (00-23 instead of 1-12)
        const bool DISPLAY_24_HOUR_TIME = false;
        // set the following to true to outline screen in emulator
        const bool DISPLAY_BORDER_BOX = true;

        static Bitmap _bitmap;

        static Timer _updateClockTimer;

        static object _updateTimeLock = new object();

        static DateTime _startTime = new DateTime(2013, 01, 01, 09, 00, 00);

        public static void Main()
        {
            _bitmap = new Bitmap(Bitmap.MaxWidth, Bitmap.MaxHeight);

            // optionally set time; comment out the following line to use the current system time
            //Microsoft.SPOT.Hardware.Utility.SetLocalTime(_startTime);

            // display the time immediately
            UpdateTime(null);

            // set up timer to refresh time every minute
            DateTime currentTime = DateTime.Now;
            TimeSpan dueTime = new TimeSpan(0, 0, 0, 59 - currentTime.Second, 1000 - currentTime.Millisecond); // beginning of next minute
            TimeSpan period = new TimeSpan(0, 0, 1, 0, 0); // update time every minute
            _updateClockTimer = new Timer(UpdateTime, null, dueTime, period); // start our minute timer

            // go to sleep; time updates will happen automatically every minute
            Thread.Sleep(Timeout.Infinite);
        }

        static void UpdateTime(object state)
        {
            int hour = 0;
            int minute = 0;

            // clear our display buffer
            _bitmap.Clear();

            // grab the current time
            DateTime currentTime = DateTime.Now;

            // set our hour and minute based on 12/24 hour settinsg
            if (DISPLAY_24_HOUR_TIME)
            {
                hour = currentTime.Hour;
            }
            else
            {
                hour = currentTime.Hour % 12;
                if (hour == 0) hour = 12;
            }
            minute = currentTime.Minute;

            // calculate the time digits
            int hourTensDigit = (hour.ToString().Length == 1 ? 0 : int.Parse(hour.ToString().Substring(0, 1)));
            int hourOnesDigit = (int.Parse(hour.ToString().Substring(hour.ToString().Length - 1)));
            int minuteTensDigit = (minute.ToString().Length == 1 ? 0 : int.Parse(minute.ToString().Substring(0, 1)));
            int minuteOnesDigit = (int.Parse(minute.ToString().Substring(minute.ToString().Length - 1)));

            // display the time
            if (DISPLAY_24_HOUR_TIME || hourTensDigit != 0)
            {
                _bitmap.DrawImage(0, 0, GetDigit(hourTensDigit), 0, 0, DIGIT_WIDTH, DIGIT_HEIGHT);
            }
            _bitmap.DrawImage(DIGIT_WIDTH, 0, GetDigit(hourOnesDigit), 0, 0, DIGIT_WIDTH, DIGIT_HEIGHT);
            _bitmap.DrawImage(0, DIGIT_HEIGHT, GetDigit(minuteTensDigit), 0, 0, DIGIT_WIDTH, DIGIT_HEIGHT);
            _bitmap.DrawImage(DIGIT_WIDTH, DIGIT_HEIGHT, GetDigit(minuteOnesDigit), 0, 0, DIGIT_WIDTH, DIGIT_HEIGHT);

            // draw a border around the screen, if desired.
            if (DISPLAY_BORDER_BOX)
            {
                _bitmap.DrawRectangle(Color.White, 1, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, 0, 0, Color.White, 0, 0, Color.White, 0, 0, 0);
            }

            // flush the display buffer to the display
            _bitmap.Flush();
        }

        static Bitmap GetDigit(int digitNumber)
        {
            switch (digitNumber)
            {
                case 0:
                    return new Bitmap(Resources.GetBytes(Resources.BinaryResources._0), Bitmap.BitmapImageType.Gif);
                case 1:
                    return new Bitmap(Resources.GetBytes(Resources.BinaryResources._1), Bitmap.BitmapImageType.Gif);
                case 2:
                    return new Bitmap(Resources.GetBytes(Resources.BinaryResources._2), Bitmap.BitmapImageType.Gif);
                case 3:
                    return new Bitmap(Resources.GetBytes(Resources.BinaryResources._3), Bitmap.BitmapImageType.Gif);
                case 4:
                    return new Bitmap(Resources.GetBytes(Resources.BinaryResources._4), Bitmap.BitmapImageType.Gif);
                case 5:
                    return new Bitmap(Resources.GetBytes(Resources.BinaryResources._5), Bitmap.BitmapImageType.Gif);
                case 6:
                    return new Bitmap(Resources.GetBytes(Resources.BinaryResources._6), Bitmap.BitmapImageType.Gif);
                case 7:
                    return new Bitmap(Resources.GetBytes(Resources.BinaryResources._7), Bitmap.BitmapImageType.Gif);
                case 8:
                    return new Bitmap(Resources.GetBytes(Resources.BinaryResources._8), Bitmap.BitmapImageType.Gif);
                case 9:
                    return new Bitmap(Resources.GetBytes(Resources.BinaryResources._9), Bitmap.BitmapImageType.Gif);
                default:
                    return null;
            }
        }
    }
}
